@@ -3,6 +3,7 @@ package release
 import (
 	"archive/tar"
 	"bytes"
+	"context"
 	"encoding/json"
 	"fmt"
 	"io"
@@ -30,6 +31,7 @@ import (
 
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/util/duration"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/cli-runtime/pkg/genericclioptions"
@@ -49,8 +51,9 @@ import (
 
 func NewInfoOptions(streams genericclioptions.IOStreams) *InfoOptions {
 	return &InfoOptions{
-		IOStreams:       streams,
-		ParallelOptions: imagemanifest.ParallelOptions{MaxPerRegistry: 4},
+		IOStreams:              streams,
+		KubeTemplatePrintFlags: *genericclioptions.NewKubeTemplatePrintFlags(),
+		ParallelOptions:        imagemanifest.ParallelOptions{MaxPerRegistry: 4},
 	}
 }
 
@@ -91,20 +94,20 @@ func NewInfo(f kcmdutil.Factory, parentName string, streams genericclioptions.IO
 			the code changes that occurred between the two release arguments. This operation is slow
 			and requires sufficient disk space on the selected drive to clone all repositories.
 		`),
-		Example: templates.Examples(`
+		Example: templates.Examples(fmt.Sprintf(`
 			# Show information about the cluster's current release
-			%[1]s
+			%[1]s info
 
 			# Show the source code that comprises a release
-			%[1]s 4.2.2 --commit-urls
+			%[1]s info 4.2.2 --commit-urls
 
 			# Show the source code difference between two releases
-			%[1]s 4.2.0 4.2.2 --commits
+			%[1]s info 4.2.0 4.2.2 --commits
 
 			# Show where the images referenced by the release are located
-			%[1]s quay.io/openshift-release-dev/ocp-release:4.2.2 --pullspecs
+			%[1]s info quay.io/openshift-release-dev/ocp-release:4.2.2 --pullspecs
 
-			`),
+			`, parentName)),
 		Run: func(cmd *cobra.Command, args []string) {
 			kcmdutil.CheckErr(o.Complete(f, cmd, args))
 			kcmdutil.CheckErr(o.Validate())
@@ -114,6 +117,7 @@ func NewInfo(f kcmdutil.Factory, parentName string, streams genericclioptions.IO
 	flags := cmd.Flags()
 	o.SecurityOptions.Bind(flags)
 	o.ParallelOptions.Bind(flags)
+	o.KubeTemplatePrintFlags.AddFlags(cmd)
 
 	flags.StringVar(&o.From, "changes-from", o.From, "Show changes from this image to the requested image.")
 
@@ -125,7 +129,7 @@ func NewInfo(f kcmdutil.Factory, parentName string, streams genericclioptions.IO
 	flags.BoolVar(&o.ShowPullSpec, "pullspecs", o.ShowPullSpec, "Display the pull spec of each image instead of the digest.")
 	flags.BoolVar(&o.ShowSize, "size", o.ShowSize, "Display the size of each image including overlap.")
 	flags.StringVar(&o.ImageFor, "image-for", o.ImageFor, "Print the pull spec of the specified image or an error if it does not exist.")
-	flags.StringVarP(&o.Output, "output", "o", o.Output, "Display the release info in an alternative format: digest|json|name|pullspec.")
+	flags.StringVarP(&o.Output, "output", "o", o.Output, "Display the release info in an alternative format: digest|json|name|pullspec|template|jsonpath.")
 	flags.StringVar(&o.ChangelogDir, "changelog", o.ChangelogDir, "Generate changelog output from the git directories extracted to this path.")
 	flags.StringVar(&o.BugsDir, "bugs", o.BugsDir, "Generate bug listings from the changelogs in the git repositories extracted to this path.")
 	flags.BoolVar(&o.IncludeImages, "include-images", o.IncludeImages, "When displaying JSON output of a release output the images the release references.")
@@ -135,6 +139,7 @@ func NewInfo(f kcmdutil.Factory, parentName string, streams genericclioptions.IO
 
 type InfoOptions struct {
 	genericclioptions.IOStreams
+	genericclioptions.KubeTemplatePrintFlags
 
 	Images  []string
 	From    string
@@ -286,10 +291,11 @@ func replaceClusterSemanticArgs(f kcmdutil.Factory, args []string, semanticArgs
 	if err != nil {
 		return args, fmt.Errorf("info expects one argument, or a connection to an OpenShift 4.x server: %v", err)
 	}
-	cv, err := client.ConfigV1().ClusterVersions().Get("version", metav1.GetOptions{})
+	cv, err := client.ConfigV1().ClusterVersions().Get(context.TODO(), "version", metav1.GetOptions{})
 	if err != nil {
-		if errors.IsNotFound(err) {
-			return args, fmt.Errorf("you must be connected to an OpenShift 4.x server to fetch the current version")
+		if errors.IsNotFound(err) || errors.ReasonForError(err) == metav1.StatusReasonUnknown {
+			klog.V(2).Infof("Unable to find cluster version object from cluster: %v", err)
+			return args, fmt.Errorf("info expects one argument, or a connection to an OpenShift 4.x server")
 		}
 		return args, fmt.Errorf("info expects one argument, or a connection to an OpenShift 4.x server: %v", err)
 	}
@@ -411,10 +417,9 @@ func (o *InfoOptions) Validate() error {
 			return fmt.Errorf("--output is not supported for this mode")
 		}
 	default:
-		switch o.Output {
-		case "", "json", "pullspec", "digest", "name":
-		default:
-			return fmt.Errorf("--output only supports 'name', 'json', 'pullspec', or 'digest'")
+		output := strings.SplitN(o.Output, "=", 2)[0]
+		if len(output) > 0 && !stringArrContains(o.allowedFormats(), output) {
+			return fmt.Errorf("--output only supports %s", strings.Join(o.allowedFormats(), ", "))
 		}
 	}
 
@@ -488,6 +493,12 @@ func (o *InfoOptions) Run() error {
 	return exitErr
 }
 
+func (opt *InfoOptions) allowedFormats() []string {
+	formats := []string{"json", "pullspec", "digest", "name"}
+	formats = append(formats, opt.KubeTemplatePrintFlags.AllowedFormats()...)
+	return formats
+}
+
 func diffContents(a, b string, out io.Writer) error {
 	fmt.Fprintf(out, `To see the differences between these releases, run:
 
@@ -504,7 +515,8 @@ func (o *InfoOptions) describeImage(release *ReleaseInfo) error {
 		_, err := io.Copy(o.Out, newContentStreamForRelease(release))
 		return err
 	}
-	switch o.Output {
+	output := strings.SplitN(o.Output, "=", 2)
+	switch output[0] {
 	case "json":
 		data, err := json.MarshalIndent(release, "", "  ")
 		if err != nil {
@@ -539,7 +551,18 @@ func (o *InfoOptions) describeImage(release *ReleaseInfo) error {
 		return nil
 	case "":
 	default:
-		return fmt.Errorf("output mode only supports 'name', 'json', 'pullspec', or 'digest'")
+		p, err := o.KubeTemplatePrintFlags.ToPrinter(o.Output)
+		if genericclioptions.IsNoCompatiblePrinterError(err) {
+			return fmt.Errorf("output mode only supports %s", strings.Join(o.allowedFormats(), ", "))
+		}
+		if err != nil {
+			return err
+		}
+		data, err := json.MarshalIndent(release, "", "  ")
+		if err != nil {
+			return err
+		}
+		return p.PrintObj(&runtime.Unknown{Raw: data}, o.Out)
 	}
 	if len(o.ImageFor) > 0 {
 		spec, err := findImageSpec(release.References, o.ImageFor, release.Image)
@@ -1378,6 +1401,13 @@ func digestOrRef(ref string) string {
 	return ref
 }
 
+// replaceUnsafeInput prevents HTML blocks from being started in markdown for external
+// inputs, but allows entities and quotes
+var replaceUnsafeInput = strings.NewReplacer(
+	`<`, "&lt;",
+	`>`, "&gt;",
+)
+
 func describeChangelog(out, errOut io.Writer, diff *ReleaseDiff, dir string) error {
 	if diff.To.Digest == diff.From.Digest {
 		return fmt.Errorf("releases are identical")
@@ -1473,37 +1503,28 @@ func describeChangelog(out, errOut io.Writer, diff *ReleaseDiff, dir string) err
 				fmt.Fprintf(out, "### %s\n\n", strings.Join(change.ImagesAffected, ", "))
 			}
 			for _, commit := range commits {
-				var suffix string
+				fmt.Fprintf(out, "*")
+				for i, bug := range commit.Bugs {
+					if i == 0 {
+						fmt.Fprintf(out, " [Bug %d](%s)", bug, fmt.Sprintf("https://bugzilla.redhat.com/show_bug.cgi?id=%d", bug))
+					} else {
+						fmt.Fprintf(out, ", [%d](%s)", bug, fmt.Sprintf("https://bugzilla.redhat.com/show_bug.cgi?id=%d", bug))
+					}
+				}
+				if len(commit.Bugs) > 0 {
+					fmt.Fprintf(out, ":")
+				}
+				fmt.Fprintf(out, " %s", replaceUnsafeInput.Replace(commit.Subject))
 				switch {
 				case commit.PullRequest > 0:
-					suffix = fmt.Sprintf("[#%d](%s)", commit.PullRequest, fmt.Sprintf("https://%s%s/pull/%d", u.Host, u.Path, commit.PullRequest))
+					fmt.Fprintf(out, " [#%d](%s)", commit.PullRequest, fmt.Sprintf("https://%s%s/pull/%d", u.Host, u.Path, commit.PullRequest))
 				case u.Host == "github.com":
 					commit := commit.Commit[:8]
-					suffix = fmt.Sprintf("[%s](%s)", commit, fmt.Sprintf("https://%s%s/commit/%s", u.Host, u.Path, commit))
+					fmt.Fprintf(out, " [%s](%s)", commit, fmt.Sprintf("https://%s%s/commit/%s", u.Host, u.Path, commit))
 				default:
-					suffix = commit.Commit[:8]
-				}
-				switch {
-				case len(commit.Bugs) > 0:
-					for i, bug := range commit.Bugs {
-						if i == 0 {
-							fmt.Fprintf(out, "* [Bug %d](%s)", bug, fmt.Sprintf("https://bugzilla.redhat.com/show_bug.cgi?id=%d", bug))
-						} else {
-							fmt.Fprintf(out, ", [%d](%s)", bug, fmt.Sprintf("https://bugzilla.redhat.com/show_bug.cgi?id=%d", bug))
-						}
-						fmt.Fprintf(out,
-							": %s %s\n",
-							commit.Subject,
-							suffix,
-						)
-					}
-				default:
-					fmt.Fprintf(out,
-						"* %s %s\n",
-						commit.Subject,
-						suffix,
-					)
+					fmt.Fprintf(out, " %s", commit.Commit[:8])
 				}
+				fmt.Fprintf(out, "\n")
 			}
 			if u.Host == "github.com" {
 				fmt.Fprintf(out, "* [Full changelog](%s)\n\n", fmt.Sprintf("https://%s%s/compare/%s...%s", u.Host, u.Path, change.From, change.To))
@@ -1748,6 +1769,10 @@ func releaseDiffContentChanges(diff *ReleaseDiff) ([]CodeChange, []ImageChange,
 		oldRepo, newRepo := from.Annotations[annotationBuildSourceLocation], to.Annotations[annotationBuildSourceLocation]
 		oldCommit, newCommit := from.Annotations[annotationBuildSourceCommit], to.Annotations[annotationBuildSourceCommit]
 
+		if len(newRepo) == 0 || len(oldRepo) == 0 {
+			continue
+		}
+
 		var alternateRepos []string
 		if len(oldRepo) > 0 && oldRepo != newRepo {
 			alternateRepos = append(alternateRepos, oldRepo)
